// This vertex shader is used to do normal mapped reflectivity mapped light mapped cubic environment mapping
vs.1.1

// Get the vertex position into projection space
m4x4 oPos, v0, c0

// The first texture coordinate set gets the texture coordinate of the mesh
mov oT0.xy, v7

// The second texture coordinate gets a world space vertex to camera vector
sub r0, v0, c8
m3x3 oT1.xyz, r0, c4

// The third, fourth and fifth texture coordinate set get a normalized 3x3 tangent 
// space to world space matrix

// The mesh to tangent space matrix is orthogonal, meaning that it's transpose is it's 
// inverse (the tangent space to mesh space matrix). Multiply the tangent space to 
// mesh space matrix by the mesh space to world space matrix to get the tangent space 
// to world space matrix.
dp3 r2.x, v8, c4
dp3 r3.x, v8, c5
dp3 r4.x, v8, c6

dp3 r2.y, v9, c4
dp3 r3.y, v9, c5
dp3 r4.y, v9, c6

dp3 r2.z, v3, c4
dp3 r3.z, v3, c5
dp3 r4.z, v3, c6

// Normalize the line vectors of the matrix and pass them as texture coordinates
dp3 r2.w, r2, r2
rsq r2.w, r2.w
mul oT2.xyz, r2.xyz, r2.w

dp3 r3.w, r3, r3
rsq r3.w, r3.w
mul oT3.xyz, r3.xyz, r3.w

dp3 r4.w, r4, r4
rsq r4.w, r4.w
mul oT4.xyz, r4.xyz, r4.w


// Compute the world space light to vertex vector for the sixth texture coordinate
sub r0, v0, c9
m3x3 oT5.xyz, r0, c4

// Compute the range based attenuation factor through the following formula:
// A=LightRange / (d*16)
dp3 r0.w, r0, r0
rcp r0.w, r0.w
mul r1.w, c9.w, c9.w
mul r1.w, r1.w, c11.w
rcp r1.w, r1.w
mul r0.w, r0.w, r1.w

// Pass the material color multiplied by the range based attenuation factor and the 
// light color and pass the alpha component of the material
mul r0.rgb, r0.w, c10
mul oD0.rgb, r0.rgb, c12
mov oD0.a, c10.a
